@@ -14,6 +14,8 @@
 
 #include <time.h>
 
+#include <boost/algorithm/string.hpp>
+
 #include "common/admin_socket.h"
 #include "common/perf_counters.h"
 #include "common/Thread.h"
@@ -27,10 +29,13 @@
 #include "log/Log.h"
 #include "auth/Crypto.h"
 #include "include/str_list.h"
+#include "common/Mutex.h"
+#include "common/Cond.h"
 
 #include <iostream>
 #include <pthread.h>
-#include <semaphore.h>
+
+#include "include/Spinlock.h"
 
 using ceph::HeartbeatMap;
 
@@ -38,30 +43,28 @@ class CephContextServiceThread : public Thread
 {
 public:
   CephContextServiceThread(CephContext *cct)
-    : _reopen_logs(false), _exit_thread(false), _cct(cct)
+    : _lock("CephContextServiceThread::_lock"),
+      _reopen_logs(false), _exit_thread(false), _cct(cct)
   {
-    sem_init(&_sem, 0, 0);
-  };
+  }
 
-  ~CephContextServiceThread()
-  {
-    sem_destroy(&_sem);
-  };
+  ~CephContextServiceThread() {}
 
   void *entry()
   {
     while (1) {
+      Mutex::Locker l(_lock);
+
       if (_cct->_conf->heartbeat_interval) {
-        struct timespec timeout;
-        clock_gettime(CLOCK_REALTIME, &timeout);
-        timeout.tv_sec += _cct->_conf->heartbeat_interval;
-        sem_timedwait(&_sem, &timeout);
-      } else {
-        sem_wait(&_sem);
-      }
+        utime_t interval(_cct->_conf->heartbeat_interval, 0);
+        _cond.WaitInterval(_cct, _lock, interval);
+      } else
+        _cond.Wait(_lock);
+
       if (_exit_thread) {
         break;
       }
+
       if (_reopen_logs) {
         _cct->_log->reopen_log_file();
         _reopen_logs = false;
@@ -73,20 +76,23 @@ public:
 
   void reopen_logs()
   {
+    Mutex::Locker l(_lock);
     _reopen_logs = true;
-    sem_post(&_sem);
+    _cond.Signal();
   }
 
   void exit_thread()
   {
+    Mutex::Locker l(_lock);
     _exit_thread = true;
-    sem_post(&_sem);
+    _cond.Signal();
   }
 
 private:
-  volatile bool _reopen_logs;
-  volatile bool _exit_thread;
-  sem_t _sem;
+  Mutex _lock;
+  Cond _cond;
+  bool _reopen_logs;
+  bool _exit_thread;
   CephContext *_cct;
 };
 
@@ -149,6 +155,58 @@ public:
 };
 
 
+// cct config watcher
+class CephContextObs : public md_config_obs_t {
+  CephContext *cct;
+
+public:
+  CephContextObs(CephContext *cct) : cct(cct) {}
+
+  const char** get_tracked_conf_keys() const {
+    static const char *KEYS[] = {
+      "enable_experimental_unrecoverable_data_corrupting_features",
+      NULL
+    };
+    return KEYS;
+  }
+
+  void handle_conf_change(const md_config_t *conf,
+                          const std::set <std::string> &changed) {
+    ceph_spin_lock(&cct->_feature_lock);
+    get_str_set(conf->enable_experimental_unrecoverable_data_corrupting_features,
+		cct->_experimental_features);
+    ceph_spin_unlock(&cct->_feature_lock);
+    if (!cct->_experimental_features.empty())
+      lderr(cct) << "WARNING: the following dangerous and experimental features are enabled: "
+		 << cct->_experimental_features << dendl;
+  }
+};
+
+bool CephContext::check_experimental_feature_enabled(std::string feat)
+{
+  ceph_spin_lock(&_feature_lock);
+  bool enabled = _experimental_features.count(feat);
+  ceph_spin_unlock(&_feature_lock);
+
+  if (enabled) {
+    lderr(this) << "WARNING: experimental feature '" << feat << "' is enabled" << dendl;
+    lderr(this) << "Please be aware that this feature is experimental, untested," << dendl;
+    lderr(this) << "unsupported, and may result in data corruption, data loss," << dendl;
+    lderr(this) << "and/or irreparable damage to your cluster.  Do not use" << dendl;
+    lderr(this) << "feature with important data." << dendl;
+  } else {
+    lderr(this) << "*** experimental feature '" << feat << "' is not enabled ***" << dendl;
+    lderr(this) << "This feature is marked as experimental, which means it" << dendl;
+    lderr(this) << " - is untested" << dendl;
+    lderr(this) << " - is unsupported" << dendl;
+    lderr(this) << " - may corrupt your data" << dendl;
+    lderr(this) << " - may break your cluster is an unrecoverable fashion" << dendl;
+    lderr(this) << "To enable this feature, add this to your ceph.conf:" << dendl;
+    lderr(this) << "  enable experimental unrecoverable data corrupting features = " << feat << dendl;
+  }
+  return enabled;
+}
+
 // perfcounter hooks
 
 class CephContextHook : public AdminSocketHook {
@@ -168,6 +226,8 @@ void CephContext::do_command(std::string command, cmdmap_t& cmdmap,
 			     std::string format, bufferlist *out)
 {
   Formatter *f = new_formatter(format);
+  if (!f)
+    f = new_formatter("json-pretty");
   stringstream ss;
   for (cmdmap_t::iterator it = cmdmap.begin(); it != cmdmap.end(); ++it) {
     if (it->first != "prefix") {
@@ -184,8 +244,19 @@ void CephContext::do_command(std::string command, cmdmap_t& cmdmap,
     command == "perf schema") {
     _perf_counters_collection->dump_formatted(f, true);
   }
+  else if (command == "perf reset") {
+    std::string var;
+    if (!cmd_getval(this, cmdmap, "var", var)) {
+      f->dump_string("error", "syntax error: 'perf reset <var>'");
+    } else {
+     if(!_perf_counters_collection->reset(var))
+        f->dump_stream("error") << "Not find: " << var;
+    }
+  }
   else {
-    f->open_object_section(command.c_str());
+    string section = command;
+    boost::replace_all(section, " ", "_");
+    f->open_object_section(section.c_str());
     if (command == "config show") {
       _conf->show_config(f);
     }
@@ -223,6 +294,38 @@ void CephContext::do_command(std::string command, cmdmap_t& cmdmap,
 	    f->dump_string(var.c_str(), buf);
 	}
       }
+    } else if (command == "config diff") {
+      md_config_t def_conf;
+      def_conf.set_val("cluster", _conf->cluster);
+      def_conf.name = _conf->name;
+      def_conf.set_val("host", _conf->host);
+      def_conf.apply_changes(NULL);
+
+      map<string,pair<string,string> > diff;
+      set<string> unknown;
+      def_conf.diff(_conf, &diff, &unknown);
+      f->open_object_section("diff");
+
+      f->open_object_section("current");
+      for (map<string,pair<string,string> >::iterator p = diff.begin();
+           p != diff.end(); ++p) {
+        f->dump_string(p->first.c_str(), p->second.second);
+      }
+      f->close_section(); // current
+      f->open_object_section("defaults");
+      for (map<string,pair<string,string> >::iterator p = diff.begin();
+           p != diff.end(); ++p) {
+        f->dump_string(p->first.c_str(), p->second.first);
+      }
+      f->close_section(); // defaults
+      f->close_section(); // diff
+
+      f->open_array_section("unknown");
+      for (set<string>::iterator p = unknown.begin();
+           p != unknown.end(); ++p) {
+        f->dump_string("option", *p);
+      }
+      f->close_section(); // unknown
     } else if (command == "log flush") {
       _log->flush();
     }
@@ -241,7 +344,7 @@ void CephContext::do_command(std::string command, cmdmap_t& cmdmap,
   delete f;
   lgeneric_dout(this, 1) << "do_command '" << command << "' '" << ss.str()
 		         << "result is " << out->length() << " bytes" << dendl;
-};
+}
 
 
 CephContext::CephContext(uint32_t module_type_)
@@ -258,7 +361,9 @@ CephContext::CephContext(uint32_t module_type_)
     _crypto_none(NULL),
     _crypto_aes(NULL)
 {
-  pthread_spin_init(&_service_thread_lock, PTHREAD_PROCESS_SHARED);
+  ceph_spin_init(&_service_thread_lock);
+  ceph_spin_init(&_associated_objs_lock);
+  ceph_spin_init(&_feature_lock);
 
   _log = new ceph::log::Log(&_conf->subsys);
   _log->start();
@@ -266,6 +371,9 @@ CephContext::CephContext(uint32_t module_type_)
   _log_obs = new LogObs(_log);
   _conf->add_observer(_log_obs);
 
+  _cct_obs = new CephContextObs(this);
+  _conf->add_observer(_cct_obs);
+
   _perf_counters_collection = new PerfCountersCollection(this);
   _admin_socket = new AdminSocket(this);
   _heartbeat_map = new HeartbeatMap(this);
@@ -277,9 +385,13 @@ CephContext::CephContext(uint32_t module_type_)
   _admin_socket->register_command("perfcounters_schema", "perfcounters_schema", _admin_hook, "");
   _admin_socket->register_command("2", "2", _admin_hook, "");
   _admin_socket->register_command("perf schema", "perf schema", _admin_hook, "dump perfcounters schema");
+  _admin_socket->register_command("perf reset", "perf reset name=var,type=CephString", _admin_hook, "perf reset <name>: perf reset all or one perfcounter name");
   _admin_socket->register_command("config show", "config show", _admin_hook, "dump current config settings");
   _admin_socket->register_command("config set", "config set name=var,type=CephString name=val,type=CephString,n=N",  _admin_hook, "config set <field> <val> [<val> ...]: set a config variable");
   _admin_socket->register_command("config get", "config get name=var,type=CephString", _admin_hook, "config get <field>: get the config value");
+  _admin_socket->register_command("config diff",
+      "config diff", _admin_hook,
+      "dump diff of current config and default config");
   _admin_socket->register_command("log flush", "log flush", _admin_hook, "flush log entries to log file");
   _admin_socket->register_command("log dump", "log dump", _admin_hook, "dump recent log entries to log file");
   _admin_socket->register_command("log reopen", "log reopen", _admin_hook, "reopen log file");
@@ -290,21 +402,27 @@ CephContext::CephContext(uint32_t module_type_)
 
 CephContext::~CephContext()
 {
+  join_service_thread();
+
+  for (map<string, AssociatedSingletonObject*>::iterator it = _associated_objs.begin();
+       it != _associated_objs.end(); ++it)
+    delete it->second;
+
   if (_conf->lockdep) {
     lockdep_unregister_ceph_context(this);
   }
 
-  join_service_thread();
-
   _admin_socket->unregister_command("perfcounters_dump");
   _admin_socket->unregister_command("perf dump");
   _admin_socket->unregister_command("1");
   _admin_socket->unregister_command("perfcounters_schema");
   _admin_socket->unregister_command("perf schema");
   _admin_socket->unregister_command("2");
+  _admin_socket->unregister_command("perf reset");
   _admin_socket->unregister_command("config show");
   _admin_socket->unregister_command("config set");
   _admin_socket->unregister_command("config get");
+  _admin_socket->unregister_command("config diff");
   _admin_socket->unregister_command("log flush");
   _admin_socket->unregister_command("log dump");
   _admin_socket->unregister_command("log reopen");
@@ -323,12 +441,18 @@ CephContext::~CephContext()
   delete _log_obs;
   _log_obs = NULL;
 
+  _conf->remove_observer(_cct_obs);
+  delete _cct_obs;
+  _cct_obs = NULL;
+
   _log->stop();
   delete _log;
   _log = NULL;
 
   delete _conf;
-  pthread_spin_destroy(&_service_thread_lock);
+  ceph_spin_destroy(&_service_thread_lock);
+  ceph_spin_destroy(&_associated_objs_lock);
+  ceph_spin_destroy(&_feature_lock);
 
   delete _crypto_none;
   delete _crypto_aes;
@@ -336,14 +460,14 @@ CephContext::~CephContext()
 
 void CephContext::start_service_thread()
 {
-  pthread_spin_lock(&_service_thread_lock);
+  ceph_spin_lock(&_service_thread_lock);
   if (_service_thread) {
-    pthread_spin_unlock(&_service_thread_lock);
+    ceph_spin_unlock(&_service_thread_lock);
     return;
   }
   _service_thread = new CephContextServiceThread(this);
   _service_thread->create();
-  pthread_spin_unlock(&_service_thread_lock);
+  ceph_spin_unlock(&_service_thread_lock);
 
   // make logs flush on_exit()
   if (_conf->log_flush_on_exit)
@@ -361,22 +485,22 @@ void CephContext::start_service_thread()
 
 void CephContext::reopen_logs()
 {
-  pthread_spin_lock(&_service_thread_lock);
+  ceph_spin_lock(&_service_thread_lock);
   if (_service_thread)
     _service_thread->reopen_logs();
-  pthread_spin_unlock(&_service_thread_lock);
+  ceph_spin_unlock(&_service_thread_lock);
 }
 
 void CephContext::join_service_thread()
 {
-  pthread_spin_lock(&_service_thread_lock);
+  ceph_spin_lock(&_service_thread_lock);
   CephContextServiceThread *thread = _service_thread;
   if (!thread) {
-    pthread_spin_unlock(&_service_thread_lock);
+    ceph_spin_unlock(&_service_thread_lock);
     return;
   }
   _service_thread = NULL;
-  pthread_spin_unlock(&_service_thread_lock);
+  ceph_spin_unlock(&_service_thread_lock);
 
   thread->exit_thread();
   thread->join();
