@@ -24,17 +24,20 @@
 
 template <class K, class V>
 class SharedLRU {
+  CephContext *cct;
   typedef ceph::shared_ptr<V> VPtr;
   typedef ceph::weak_ptr<V> WeakVPtr;
   Mutex lock;
   size_t max_size;
   Cond cond;
   unsigned size;
-
+public:
+  int waiting;
+private:
   map<K, typename list<pair<K, VPtr> >::iterator > contents;
   list<pair<K, VPtr> > lru;
 
-  map<K, WeakVPtr> weak_refs;
+  map<K, pair<WeakVPtr, V*> > weak_refs;
 
   void trim_cache(list<VPtr> *to_release) {
     while (size > max_size) {
@@ -43,7 +46,7 @@ class SharedLRU {
     }
   }
 
-  void lru_remove(K key) {
+  void lru_remove(const K& key) {
     typename map<K, typename list<pair<K, VPtr> >::iterator>::iterator i =
       contents.find(key);
     if (i == contents.end())
@@ -53,7 +56,7 @@ class SharedLRU {
     contents.erase(i);
   }
 
-  void lru_add(K key, VPtr val, list<VPtr> *to_release) {
+  void lru_add(const K& key, const VPtr& val, list<VPtr> *to_release) {
     typename map<K, typename list<pair<K, VPtr> >::iterator>::iterator i =
       contents.find(key);
     if (i != contents.end()) {
@@ -66,9 +69,12 @@ class SharedLRU {
     }
   }
 
-  void remove(K key) {
+  void remove(const K& key, V *valptr) {
     Mutex::Locker l(lock);
-    weak_refs.erase(key);
+    typename map<K, pair<WeakVPtr, V*> >::iterator i = weak_refs.find(key);
+    if (i != weak_refs.end() && i->second.second == valptr) {
+      weak_refs.erase(i);
+    }
     cond.Signal();
   }
 
@@ -78,32 +84,71 @@ class SharedLRU {
     K key;
     Cleanup(SharedLRU<K, V> *cache, K key) : cache(cache), key(key) {}
     void operator()(V *ptr) {
-      cache->remove(key);
+      cache->remove(key, ptr);
       delete ptr;
     }
   };
 
 public:
-  SharedLRU(size_t max_size = 20)
-    : lock("SharedLRU::lock"), max_size(max_size), size(0) {}
+  SharedLRU(CephContext *cct = NULL, size_t max_size = 20)
+    : cct(cct), lock("SharedLRU::lock"), max_size(max_size), 
+      size(0), waiting(0) {}
   
   ~SharedLRU() {
     contents.clear();
     lru.clear();
-    assert(weak_refs.empty());
+    if (!weak_refs.empty()) {
+      lderr(cct) << "leaked refs:\n";
+      dump_weak_refs(*_dout);
+      *_dout << dendl;
+      assert(weak_refs.empty());
+    }
   }
 
-  void clear(K key) {
+  void set_cct(CephContext *c) {
+    cct = c;
+  }
+
+  void dump_weak_refs() {
+    lderr(cct) << "leaked refs:\n";
+    dump_weak_refs(*_dout);
+    *_dout << dendl;
+  }
+
+  void dump_weak_refs(ostream& out) {
+    for (typename map<K, pair<WeakVPtr, V*> >::iterator p = weak_refs.begin();
+	 p != weak_refs.end();
+	 ++p) {
+      out << __func__ << " " << this << " weak_refs: "
+	  << p->first << " = " << p->second.second
+	  << " with " << p->second.first.use_count() << " refs"
+	  << std::endl;
+    }
+  }
+
+  void clear(const K& key) {
     VPtr val; // release any ref we have after we drop the lock
     {
       Mutex::Locker l(lock);
       if (weak_refs.count(key)) {
-	val = weak_refs[key].lock();
+	val = weak_refs[key].first.lock();
       }
       lru_remove(key);
     }
   }
 
+  void purge(const K &key) {
+    VPtr val; // release any ref we have after we drop the lock
+    {
+      Mutex::Locker l(lock);
+      if (weak_refs.count(key)) {
+	val = weak_refs[key].first.lock();
+      }
+      lru_remove(key);
+      weak_refs.erase(key);
+    }
+  }
+
   void set_size(size_t new_size) {
     list<VPtr> to_release;
     {
@@ -119,20 +164,22 @@ public:
     return weak_refs.begin()->first;
   }
 
-  VPtr lower_bound(K key) {
+  VPtr lower_bound(const K& key) {
     VPtr val;
     list<VPtr> to_release;
     {
       Mutex::Locker l(lock);
+      ++waiting;
       bool retry = false;
       do {
 	retry = false;
 	if (weak_refs.empty())
 	  break;
-	typename map<K, WeakVPtr>::iterator i = weak_refs.lower_bound(key);
+	typename map<K, pair<WeakVPtr, V*> >::iterator i =
+	  weak_refs.lower_bound(key);
 	if (i == weak_refs.end())
 	  --i;
-	val = i->second.lock();
+	val = i->second.first.lock();
 	if (val) {
 	  lru_add(i->first, val, &to_release);
 	} else {
@@ -141,20 +188,23 @@ public:
 	if (retry)
 	  cond.Wait(lock);
       } while (retry);
+      --waiting;
     }
     return val;
   }
 
-  VPtr lookup(K key) {
+  VPtr lookup(const K& key) {
     VPtr val;
     list<VPtr> to_release;
     {
       Mutex::Locker l(lock);
+      ++waiting;
       bool retry = false;
       do {
 	retry = false;
-	if (weak_refs.count(key)) {
-	  val = weak_refs[key].lock();
+	typename map<K, pair<WeakVPtr, V*> >::iterator i = weak_refs.find(key);
+	if (i != weak_refs.end()) {
+	  val = i->second.first.lock();
 	  if (val) {
 	    lru_add(key, val, &to_release);
 	  } else {
@@ -164,20 +214,48 @@ public:
 	if (retry)
 	  cond.Wait(lock);
       } while (retry);
+      --waiting;
     }
     return val;
   }
 
-  VPtr add(K key, V *value) {
-    VPtr val(value, Cleanup(this, key));
+  /***
+   * Inserts a key if not present, or bumps it to the front of the LRU if
+   * it is, and then gives you a reference to the value. If the key already
+   * existed, you are responsible for deleting the new value you tried to
+   * insert.
+   *
+   * @param key The key to insert
+   * @param value The value that goes with the key
+   * @param existed Set to true if the value was already in the
+   * map, false otherwise
+   * @return A reference to the map's value for the given key
+   */
+  VPtr add(const K& key, V *value, bool *existed = NULL) {
+    VPtr val;
     list<VPtr> to_release;
     {
       Mutex::Locker l(lock);
-      weak_refs.insert(make_pair(key, val));
+      typename map<K, pair<WeakVPtr, V*> >::iterator actual =
+	weak_refs.lower_bound(key);
+      if (actual != weak_refs.end() && actual->first == key) {
+        if (existed) 
+          *existed = true;
+
+        return actual->second.first.lock();
+      }
+
+      if (existed)      
+        *existed = false;
+
+      val = VPtr(value, Cleanup(this, key));
+      weak_refs.insert(actual, make_pair(key, make_pair(val, value)));
       lru_add(key, val, &to_release);
     }
     return val;
   }
+
+  friend class SharedLRUTest;
 };
 
 #endif
