@@ -3,9 +3,9 @@
 #ifndef CEPH_LIBRBD_AIOREQUEST_H
 #define CEPH_LIBRBD_AIOREQUEST_H
 
-#include <map>
+#include "include/int_types.h"
 
-#include "inttypes.h"
+#include <map>
 
 #include "common/snap_types.h"
 #include "include/buffer.h"
@@ -65,11 +65,11 @@ namespace librbd {
 	    uint64_t objectno, uint64_t offset, uint64_t len,
 	    vector<pair<uint64_t,uint64_t> >& be,
 	    librados::snap_t snap_id, bool sparse,
-	    Context *completion)
+	    Context *completion, int op_flags)
       : AioRequest(ictx, oid, objectno, offset, len, snap_id, completion,
 		   false),
-	m_buffer_extents(be),
-	m_tried_parent(false), m_sparse(sparse) {
+	m_buffer_extents(be), m_tried_parent(false),
+	m_sparse(sparse), m_op_flags(op_flags) {
     }
     virtual ~AioRead() {}
     virtual bool should_complete(int r);
@@ -86,6 +86,7 @@ namespace librbd {
     vector<pair<uint64_t,uint64_t> > m_buffer_extents;
     bool m_tried_parent;
     bool m_sparse;
+    int m_op_flags;
   };
 
   class AbstractWrite : public AioRequest {
@@ -160,16 +161,20 @@ namespace librbd {
 		      completion, false),
 	m_write_data(data) {
       guard_write();
-      m_write.write(m_object_off, data);
+      add_write_ops(m_write);
     }
     virtual ~AioWrite() {}
 
+    void set_op_flags(int op_flags) {
+      m_write.set_op_flags2(op_flags);
+    }
   protected:
     virtual void add_copyup_ops() {
-      m_copyup.write(m_object_off, m_write_data);
+      add_write_ops(m_copyup);
     }
 
   private:
+    void add_write_ops(librados::ObjectWriteOperation &wr);
     ceph::bufferlist m_write_data;
   };
 
