@@ -358,6 +358,15 @@ namespace librbd {
     return r;
   }
 
+  int Image::is_exclusive_lock_owner(bool *is_owner)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    tracepoint(librbd, is_exclusive_lock_owner_enter, ictx);
+    int r = librbd::is_exclusive_lock_owner(ictx, is_owner);
+    tracepoint(librbd, is_exclusive_lock_owner_exit, ictx, r, *is_owner);
+    return r;
+  }
+
   int Image::copy(IoCtx& dest_io_ctx, const char *destname)
   {
     ImageCtx *ictx = (ImageCtx *)ctx;
@@ -591,7 +600,19 @@ namespace librbd {
     tracepoint(librbd, read_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only, ofs, len);
     bufferptr ptr(len);
     bl.push_back(ptr);
-    int r = librbd::read(ictx, ofs, len, bl.c_str());
+    int r = librbd::read(ictx, ofs, len, bl.c_str(), 0);
+    tracepoint(librbd, read_exit, r);
+    return r;
+  }
+
+  ssize_t Image::read2(uint64_t ofs, size_t len, bufferlist& bl, int op_flags)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    tracepoint(librbd, read2_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(),
+		ictx->read_only, ofs, len, op_flags);
+    bufferptr ptr(len);
+    bl.push_back(ptr);
+    int r = librbd::read(ictx, ofs, len, bl.c_str(), op_flags);
     tracepoint(librbd, read_exit, r);
     return r;
   }
@@ -640,7 +661,21 @@ namespace librbd {
       tracepoint(librbd, write_exit, -EINVAL);
       return -EINVAL;
     }
-    int r = librbd::write(ictx, ofs, len, bl.c_str());
+    int r = librbd::write(ictx, ofs, len, bl.c_str(), 0);
+    tracepoint(librbd, write_exit, r);
+    return r;
+  }
+
+   ssize_t Image::write2(uint64_t ofs, size_t len, bufferlist& bl, int op_flags)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    tracepoint(librbd, write2_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only,
+		ofs, len, bl.length() < len ? NULL : bl.c_str(), op_flags);
+    if (bl.length() < len) {
+      tracepoint(librbd, write_exit, -EINVAL);
+      return -EINVAL;
+    }
+    int r = librbd::write(ictx, ofs, len, bl.c_str(), op_flags);
     tracepoint(librbd, write_exit, r);
     return r;
   }
@@ -664,7 +699,23 @@ namespace librbd {
       return -EINVAL;
     }
     int r = librbd::aio_write(ictx, off, len, bl.c_str(),
-			     (librbd::AioCompletion *)c->pc);
+			     (librbd::AioCompletion *)c->pc, 0);
+    tracepoint(librbd, aio_write_exit, r);
+    return r;
+  }
+
+  int Image::aio_write2(uint64_t off, size_t len, bufferlist& bl,
+			  RBD::AioCompletion *c, int op_flags)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    tracepoint(librbd, aio_write2_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(),
+		ictx->read_only, off, len, bl.length() < len ? NULL : bl.c_str(), c->pc, op_flags);
+    if (bl.length() < len) {
+      tracepoint(librbd, aio_write_exit, -EINVAL);
+      return -EINVAL;
+    }
+    int r = librbd::aio_write(ictx, off, len, bl.c_str(),
+			     (librbd::AioCompletion *)c->pc, op_flags);
     tracepoint(librbd, aio_write_exit, r);
     return r;
   }
@@ -685,7 +736,20 @@ namespace librbd {
     tracepoint(librbd, aio_read_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only, off, len, bl.c_str(), c->pc);
     ldout(ictx->cct, 10) << "Image::aio_read() buf=" << (void *)bl.c_str() << "~"
 			 << (void *)(bl.c_str() + len - 1) << dendl;
-    int r = librbd::aio_read(ictx, off, len, NULL, &bl, (librbd::AioCompletion *)c->pc);
+    int r = librbd::aio_read(ictx, off, len, NULL, &bl, (librbd::AioCompletion *)c->pc, 0);
+    tracepoint(librbd, aio_read_exit, r);
+    return r;
+  }
+
+  int Image::aio_read2(uint64_t off, size_t len, bufferlist& bl,
+			RBD::AioCompletion *c, int op_flags)
+  {
+    ImageCtx *ictx = (ImageCtx *)ctx;
+    tracepoint(librbd, aio_read2_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(),
+		ictx->read_only, off, len, bl.c_str(), c->pc, op_flags);
+    ldout(ictx->cct, 10) << "Image::aio_read() buf=" << (void *)bl.c_str() << "~"
+			 << (void *)(bl.c_str() + len - 1) << dendl;
+    int r = librbd::aio_read(ictx, off, len, NULL, &bl, (librbd::AioCompletion *)c->pc, op_flags);
     tracepoint(librbd, aio_read_exit, r);
     return r;
   }
@@ -1108,6 +1172,17 @@ extern "C" int rbd_get_parent_info(rbd_image_t image,
   return 0;
 }
 
+extern "C" int rbd_is_exclusive_lock_owner(rbd_image_t image, int *is_owner)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  tracepoint(librbd, is_exclusive_lock_owner_enter, ictx);
+  bool owner;
+  int r = librbd::is_exclusive_lock_owner(ictx, &owner);
+  *is_owner = owner ? 1 : 0;
+  tracepoint(librbd, is_exclusive_lock_owner_exit, ictx, r, *is_owner);
+  return r;
+}
+
 /* snapshots */
 extern "C" int rbd_snap_create(rbd_image_t image, const char *snap_name)
 {
@@ -1416,11 +1491,23 @@ extern "C" ssize_t rbd_read(rbd_image_t image, uint64_t ofs, size_t len,
 {
   librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
   tracepoint(librbd, read_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only, ofs, len);
-  int r = librbd::read(ictx, ofs, len, buf);
+  int r = librbd::read(ictx, ofs, len, buf, 0);
   tracepoint(librbd, read_exit, r);
   return r;
 }
 
+extern "C" ssize_t rbd_read2(rbd_image_t image, uint64_t ofs, size_t len,
+			      char *buf, int op_flags)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  tracepoint(librbd, read2_enter, ictx, ictx->name.c_str(),
+	      ictx->snap_name.c_str(), ictx->read_only, ofs, len, op_flags);
+  int r = librbd::read(ictx, ofs, len, buf, op_flags);
+  tracepoint(librbd, read_exit, r);
+  return r;
+}
+
+
 extern "C" int64_t rbd_read_iterate(rbd_image_t image, uint64_t ofs, size_t len,
 				    int (*cb)(uint64_t, size_t, const char *, void *),
 				    void *arg)
@@ -1463,11 +1550,23 @@ extern "C" ssize_t rbd_write(rbd_image_t image, uint64_t ofs, size_t len,
 {
   librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
   tracepoint(librbd, write_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only, ofs, len, buf);
-  int r = librbd::write(ictx, ofs, len, buf);
+  int r = librbd::write(ictx, ofs, len, buf, 0);
   tracepoint(librbd, write_exit, r);
   return r;
 }
 
+extern "C" ssize_t rbd_write2(rbd_image_t image, uint64_t ofs, size_t len,
+			      const char *buf, int op_flags)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  tracepoint(librbd, write2_enter, ictx, ictx->name.c_str(),
+	      ictx->snap_name.c_str(), ictx->read_only, ofs, len, buf, op_flags);
+  int r = librbd::write(ictx, ofs, len, buf, op_flags);
+  tracepoint(librbd, write_exit, r);
+  return r;
+}
+
+
 extern "C" int rbd_discard(rbd_image_t image, uint64_t ofs, uint64_t len)
 {
   librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
@@ -1494,11 +1593,25 @@ extern "C" int rbd_aio_write(rbd_image_t image, uint64_t off, size_t len,
   librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
   tracepoint(librbd, aio_write_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only, off, len, buf, comp->pc);
   int r = librbd::aio_write(ictx, off, len, buf,
-			   (librbd::AioCompletion *)comp->pc);
+			   (librbd::AioCompletion *)comp->pc, 0);
   tracepoint(librbd, aio_write_exit, r);
   return r;
 }
 
+extern "C" int rbd_aio_write2(rbd_image_t image, uint64_t off, size_t len,
+			      const char *buf, rbd_completion_t c, int op_flags)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
+  tracepoint(librbd, aio_write2_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(),
+	      ictx->read_only, off, len, buf, comp->pc, op_flags);
+  int r = librbd::aio_write(ictx, off, len, buf,
+			   (librbd::AioCompletion *)comp->pc, op_flags);
+  tracepoint(librbd, aio_write_exit, r);
+  return r;
+}
+
+
 extern "C" int rbd_aio_discard(rbd_image_t image, uint64_t off, uint64_t len,
 			       rbd_completion_t c)
 {
@@ -1517,7 +1630,20 @@ extern "C" int rbd_aio_read(rbd_image_t image, uint64_t off, size_t len,
   librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
   tracepoint(librbd, aio_read_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(), ictx->read_only, off, len, buf, comp->pc);
   int r = librbd::aio_read(ictx, off, len, buf, NULL,
-			  (librbd::AioCompletion *)comp->pc);
+			  (librbd::AioCompletion *)comp->pc, 0);
+  tracepoint(librbd, aio_read_exit, r);
+  return r;
+}
+
+extern "C" int rbd_aio_read2(rbd_image_t image, uint64_t off, size_t len,
+			      char *buf, rbd_completion_t c, int op_flags)
+{
+  librbd::ImageCtx *ictx = (librbd::ImageCtx *)image;
+  librbd::RBD::AioCompletion *comp = (librbd::RBD::AioCompletion *)c;
+  tracepoint(librbd, aio_read2_enter, ictx, ictx->name.c_str(), ictx->snap_name.c_str(),
+	      ictx->read_only, off, len, buf, comp->pc, op_flags);
+  int r = librbd::aio_read(ictx, off, len, buf, NULL,
+			  (librbd::AioCompletion *)comp->pc, op_flags);
   tracepoint(librbd, aio_read_exit, r);
   return r;
 }
