@@ -15,7 +15,9 @@
 #include "ReplicatedBackend.h"
 #include "messages/MOSDOp.h"
 #include "messages/MOSDSubOp.h"
+#include "messages/MOSDRepOp.h"
 #include "messages/MOSDSubOpReply.h"
+#include "messages/MOSDRepOpReply.h"
 #include "messages/MOSDPGPush.h"
 #include "messages/MOSDPGPull.h"
 #include "messages/MOSDPGPushReply.h"
@@ -163,6 +165,11 @@ bool ReplicatedBackend::handle_message(
     break;
   }
 
+  case MSG_OSD_REPOP: {
+    sub_op_modify(op);
+    return true;
+  }
+
   case MSG_OSD_SUBOPREPLY: {
     MOSDSubOpReply *r = static_cast<MOSDSubOpReply*>(op->get_req());
     if (r->ops.size() >= 1) {
@@ -173,13 +180,19 @@ bool ReplicatedBackend::handle_message(
 	sub_op_push_reply(op);
 	return true;
       }
-    } else {
-      sub_op_modify_reply(op);
+    }
+    else {
+      sub_op_modify_reply<MOSDSubOpReply, MSG_OSD_SUBOPREPLY>(op);
       return true;
     }
     break;
   }
 
+  case MSG_OSD_REPOPREPLY: {
+    sub_op_modify_reply<MOSDRepOpReply, MSG_OSD_REPOPREPLY>(op);
+    return true;
+  }
+
   default:
     break;
   }
@@ -300,9 +313,11 @@ class RPGTransaction : public PGBackend::PGTransaction {
       return coll;
   }
 public:
-  RPGTransaction(coll_t coll, coll_t temp_coll)
+  RPGTransaction(coll_t coll, coll_t temp_coll, bool use_tbl)
     : coll(coll), temp_coll(temp_coll), t(new ObjectStore::Transaction), written(0)
-    {}
+    {
+      t->set_use_tbl(use_tbl);
+    }
 
   /// Yields ownership of contained transaction
   ObjectStore::Transaction *get_transaction() {
@@ -476,7 +491,7 @@ public:
 
 PGBackend::PGTransaction *ReplicatedBackend::get_transaction()
 {
-  return new RPGTransaction(coll, get_temp_coll());
+  return new RPGTransaction(coll, get_temp_coll(), parent->transaction_use_tbl());
 }
 
 class C_OSD_OnOpCommit : public Context {
@@ -557,6 +572,7 @@ void ReplicatedBackend::submit_transaction(
     op_t);
 
   ObjectStore::Transaction local_t;
+  local_t.set_use_tbl(op_t->get_use_tbl());
   if (t->get_temp_added().size()) {
     get_temp_coll(&local_t);
     add_temp_objs(t->get_temp_added());
@@ -570,8 +586,7 @@ void ReplicatedBackend::submit_transaction(
     trim_rollback_to,
     true,
     &local_t);
-  local_t.append(*op_t);
-  local_t.swap(*op_t);
+  (*op_t).append(local_t);
   
   op_t->register_on_applied_sync(on_local_applied_sync);
   op_t->register_on_applied(
@@ -626,10 +641,12 @@ void ReplicatedBackend::op_commit(
   }
 }
 
+template<typename T, int MSGTYPE>
 void ReplicatedBackend::sub_op_modify_reply(OpRequestRef op)
 {
-  MOSDSubOpReply *r = static_cast<MOSDSubOpReply*>(op->get_req());
-  assert(r->get_type() == MSG_OSD_SUBOPREPLY);
+  T *r = static_cast<T *>(op->get_req());
+  assert(r->get_header().type == MSGTYPE);
+  assert(MSGTYPE == MSG_OSD_SUBOPREPLY || MSGTYPE == MSG_OSD_REPOPREPLY);
 
   op->mark_started();
 
@@ -693,9 +710,12 @@ void ReplicatedBackend::sub_op_modify_reply(OpRequestRef op)
 
 void ReplicatedBackend::be_deep_scrub(
   const hobject_t &poid,
+  uint32_t seed,
   ScrubMap::object &o,
-  ThreadPool::TPHandle &handle) {
-  bufferhash h, oh;
+  ThreadPool::TPHandle &handle)
+{
+  dout(10) << __func__ << " " << poid << " seed " << seed << dendl;
+  bufferhash h(seed), oh(seed);
   bufferlist bl, hdrbl;
   int r;
   __u64 pos = 0;
@@ -712,7 +732,7 @@ void ReplicatedBackend::be_deep_scrub(
     bl.clear();
   }
   if (r == -EIO) {
-    dout(25) << "_scan_list  " << poid << " got "
+    dout(25) << __func__ << "  " << poid << " got "
 	     << r << " on read, read_error" << dendl;
     o.read_error = true;
   }
@@ -725,14 +745,21 @@ void ReplicatedBackend::be_deep_scrub(
     ghobject_t(
       poid, ghobject_t::NO_GEN, get_parent()->whoami_shard().shard),
     &hdrbl, true);
-  if (r == 0) {
+  // NOTE: bobtail to giant, we would crc the head as (len, head).
+  // that changes at the same time we start using a non-zero seed.
+  if (r == 0 && hdrbl.length()) {
     dout(25) << "CRC header " << string(hdrbl.c_str(), hdrbl.length())
              << dendl;
-    ::encode(hdrbl, bl);
-    oh << bl;
-    bl.clear();
+    if (seed == 0) {
+      // legacy
+      bufferlist bl;
+      ::encode(hdrbl, bl);
+      oh << bl;
+    } else {
+      oh << hdrbl;
+    }
   } else if (r == -EIO) {
-    dout(25) << "_scan_list  " << poid << " got "
+    dout(25) << __func__ << "  " << poid << " got "
 	     << r << " on omap header read, read_error" << dendl;
     o.read_error = true;
   }
@@ -759,7 +786,7 @@ void ReplicatedBackend::be_deep_scrub(
     bl.clear();
   }
   if (iter->status() == -EIO) {
-    dout(25) << "_scan_list  " << poid << " got "
+    dout(25) << __func__ << "  " << poid << " got "
 	     << r << " on omap scan, read_error" << dendl;
     o.read_error = true;
   }
