@@ -1,6 +1,7 @@
 #!/bin/bash
 #
 # Copyright (C) 2014 Cloudwatt <libre.licensing@cloudwatt.com>
+# Copyright (C) 2014 Red Hat <contact@redhat.com>
 #
 # Author: Loic Dachary <loic@dachary.org>
 #
@@ -19,14 +20,15 @@ source test/mon/mon-test-helpers.sh
 function run() {
     local dir=$1
 
+    export CEPH_MON="127.0.0.1:7104"
     export CEPH_ARGS
     CEPH_ARGS+="--fsid=$(uuidgen) --auth-supported=none "
-    CEPH_ARGS+="--mon-host=127.0.0.1 "
+    CEPH_ARGS+="--mon-host=$CEPH_MON "
 
     FUNCTIONS=${FUNCTIONS:-$(set | sed -n -e 's/^\(TEST_[0-9a-z_]*\) .*/\1/p')}
     for TEST_function in $FUNCTIONS ; do
 	setup $dir || return 1
-	run_mon $dir a --public-addr 127.0.0.1
+	run_mon $dir a --public-addr $CEPH_MON
 	if ! $TEST_function $dir ; then
 	  cat $dir/a/log
 	  return 1
@@ -175,6 +177,17 @@ EOF
 
 }
 
+function TEST_crush_rename_bucket() {
+    local dir=$1
+
+    ./ceph osd crush add-bucket host1 host
+    ! ./ceph osd tree | grep host2 || return 1
+    ./ceph osd crush rename-bucket host1 host2 || return 1
+    ./ceph osd tree | grep host2 || return 1
+    ./ceph osd crush rename-bucket host1 host2 || return 1 # idempotency
+    ./ceph osd crush rename-bucket nonexistent something 2>&1 | grep "Error ENOENT" || return 1
+}
+
 main osd-crush
 
 # Local Variables:
